@@ -86,7 +86,8 @@ prepareAlias <- function(alias) {
       vi <- as.character(vi)
     }
     if (!is.character(vi)) {
-      stop('wrapr:let alias values must all be strings or names')
+      stop(paste('wrapr:let alias values must all be strings or names (',
+                 ni,'is class:', paste(class(vi), collapse=', '), ')'))
     }
     if (length(vi) != 1) {
       stop('wrapr:let alias values must all be single strings (not arrays or null)')
@@ -97,51 +98,56 @@ prepareAlias <- function(alias) {
     if (!isValidAndUnreservedName(vi)) {
         stop(paste('wrapr:let alias value not a valid name: "', vi, '"'))
     }
-    if(vi!=ni) {
-      if(vi %in% names(alias)) {
-        stop("wrapr::prepareAlias except for identity assignments keys and destinations must be disjoint")
-      }
-    }
   }
   alias <- lapply(alias, as.character)
   alias
 }
 
-#' Substitute text.
+#' Substitute text (note text can be a vector).
 #'
 #' @param alias mapping named list/vector to strings/names or general
 #' @param strexpr character vector source text to be re-written
 #' @param ... force later arguments to be bound by name.
-#' @param debugPrint logical if TRUE print debugging information
 #' @return parsed R expression with substitutions
 #'
 #'
 #'
 letprep_str <- function(alias, strexpr,
-                    ...,
-                    debugPrint= FALSE) {
+                    ...) {
   if(length(list(...))>0) {
     stop("wrapr::letprep_str unexpected arguments.")
   }
-  alias <- prepareAlias(alias)
   if(!is.character(strexpr)) {
-    stop("wrapr::letprep_str strexpr must be length 1 character array")
+    stop("wrapr::letprep_str strexpr must be a character array")
   }
-  # re-write the parse tree and prepare for execution
   body <- strexpr
-  for (ni in names(alias)) {
-    value <- alias[[ni]]
-    if(!is.null(value)) {
-      value <- as.character(value)
-      if(ni!=value) {
-        pattern <- paste0("\\b", ni, "\\b")
-        body <- gsub(pattern, value, body)
+  if(length(alias)>0) {
+    # find a token not in alias or block
+    testText <- paste(paste(c(names(alias), as.character(alias)), collapse = ' '),
+                      strexpr)
+    tok <- "WRAPR_TOK"
+    while(length(grep(tok,testText))>0) {
+      tok <- paste0("WRAPR_TOK_",paste(sample(LETTERS, 15, replace = TRUE),
+                                       collapse = ''))
+    }
+    # re-write the parse tree and prepare for execution in 2 stages to allows swaps
+    alias1 <- paste(tok, seq_len(length(alias)), sep= '_')
+    names(alias1) <- names(alias)
+    alias2 <- as.character(alias)
+    names(alias2) <- as.character(alias1)
+    for(aliasi in list(alias1, alias2)) {
+      for (ni in names(aliasi)) {
+        value <- aliasi[[ni]]
+        if(!is.null(value)) {
+          value <- as.character(value)
+          if(ni!=value) {
+            pattern <- paste0("\\b", ni, "\\b")
+            body <- gsub(pattern, value, body)
+          }
+        }
       }
     }
   }
-  if(debugPrint) {
-    print(body)
-  }
   parse(text = body)
 }
 
@@ -158,26 +164,81 @@ letprep_str <- function(alias, strexpr,
 letprep_lang <- function(alias, lexpr) {
   nexpr <- lexpr
   n <- length(nexpr)
+  # left-hand sides of lists/calls are represented as keys
   nms <- names(nexpr)
-  for(i in seq_len(n)) {
-    ki <- as.character(nms[[i]])
-    if(length(ki)>0) {
-      ri <- alias[[ki]]
-      if((!is.null(ri))&&(ri!=ki)) {
-        nms[[i]] <- ri
+  if(length(nms)>0) {
+    for(i in seq_len(length(nms))) {
+      ki <- as.character(nms[[i]])
+      if(length(ki)>0) {
+        ri <- alias[[ki]]
+        if((length(ri)>0)&&(ri!=ki)) {
+          nms[[i]] <- ri
+        }
       }
     }
+    names(nexpr) <- nms
   }
-  names(nexpr) <- nms
-  if(is.symbol(nexpr)) {
+  # try some easy cases first
+  if(is.character(nexpr)) {
+    return(nexpr)
+  }
+  # this is the main re-mapper
+  if(is.symbol(nexpr)) { # same as is.name()
+    # symbol is not subsettable, so length==1
+    #  as.name('x')[[1]]
+    #  # Error in as.name("x")[[1]] : object of type 'symbol' is not subsettable
+    # and can't have names
+    #   names(as.name("x")) <- 'a'
+    #   ## Error in names(as.name("x")) <- "a" :
+    #   ## target of assignment expands to non-language object
     ki <- as.character(nexpr)
     ri <- alias[[ki]]
-    if((!is.null(ri))&&(ri!=ki)) {
+    if((length(ri)>0)&&(ri!=ki)) {
       return(as.name(ri))
     }
     return(nexpr)
   }
-  if(is.language(nexpr)) {
+  # just in case (establishes an invarient of n>=1)
+  if(n<=0) {
+    return(nexpr)
+  }
+  # special cases
+  if(is.call(nexpr)) {
+    callName <- as.character(nexpr[[1]])
+    if(length(callName)==1) {
+      # get into special cases,
+      #  detect them very strictly and return out of them
+      if((callName=='$') && (n==3)) {
+        # special case a$"b"
+        # let(c(x='y'), d$"x", eval=FALSE)
+        # know length should be 3 from:
+        #  do.call('$',list(data.frame(x=1:3),'x','z'))
+        #  # Error in list(x = 1:3)$x : 3 arguments passed to '$' which requires 2
+        # know the 3rd argument can be treated as a name from:
+        #  data.frame(x=1:3)$1
+        #  # Error: unexpected numeric constant in "data.frame(x=1:3)$1"
+        nexpr[[2]] <- letprep_lang(alias, nexpr[[2]])
+        nexpr[[3]] <- letprep_lang(alias, as.name(nexpr[[3]]))
+        return(nexpr)
+      }
+    }
+  }
+  if(is.expression(nexpr) || is.call(nexpr)) {
+    # from help(is.expression):
+    #   "As an object of mode "expression" is a list"
+    # as x[[1]] isn't x for lists  (which is not true for vectors as 1[[1]]  is 1)
+    # we know we can try to recurse without an obvious infinite loop
+    for(i in seq_len(n)) {
+      nexpr[[i]] <- letprep_lang(alias, nexpr[[i]])
+    }
+    return(nexpr)
+  }
+  # should be done as is.language() is supposed to imply one of:
+  #   is.name(), is.call(), or is.expr()
+  # the recurse, assuming language objects are list-like in that
+  #  x[[1]] isn't x (which is not true for vectors as 1[[1]]  is 1).
+  # was recursing on is.language(), now do it on n>1
+  if(n>1) {
     for(i in seq_len(n)) {
       nexpr[[i]] <- letprep_lang(alias, nexpr[[i]])
     }
@@ -194,7 +255,7 @@ letprep_lang <- function(alias, lexpr) {
 #' Mnemonic: "expr code symbols are on the left, external data and function argument names are on the right."
 #'
 #' Please see the \code{wrapr} \code{vignette} for some discussion of let and crossing function call boundaries: \code{vignette('wrapr','wrapr')}.
-#' Transformation is performed by substitution on the expression parse tree, so be wary of name collisions or aliasing.
+#' Transformation is performed by substitution, so please be wary of name collisions or aliasing.
 #'
 #' Something like \code{let} is only useful to get control of a function that is parameterized
 #' (in the sense it take column names) but non-standard (in that it takes column names from
@@ -205,7 +266,8 @@ letprep_lang <- function(alias, lexpr) {
 #' we can use a \code{let} helper.   \code{dplyr::mutate} is
 #' parameterized (in the sense it can work over user supplied columns and expressions), but column names are captured through non-standard evaluation
 #' (and it rapidly becomes unwieldy to use complex formulas with the standard evaluation equivalent \code{dplyr::mutate_}).
-#' \code{alias} can not include the symbol "\code{.}". Except for identity assignments keys and destinations must be disjoint.
+#' \code{alias} can not include the symbol "\code{.}".
+#'
 #'
 #' The intent from is from the user perspective to have (if
 #' \code{a <- 1; b <- 2}):
@@ -214,15 +276,21 @@ letprep_lang <- function(alias, lexpr) {
 #'
 #' \code{let} deliberately checks that it is mapping only to legal \code{R} names;
 #' this is to discourage the use of \code{let} to make names to arbitrary values, as
-#' that is the more properly left to \code{R}'s enviroment systems.
+#' that is the more properly left to \code{R}'s environment systems.
+#' \code{let} is intended to transform
+#' "tame" variable and column names to "tame" variable and column names.  Substitution
+#' outcomes that are not valid simple \code{R} variable names (produced with out use of
+#' back-ticks) are forbidden.  It is suggested that substitution targets be written
+#' \code{ALL_CAPS} style to make them stand out.
 #'
 #'
 #' @param alias mapping from free names in expr to target names to use.
 #' @param expr block to prepare for execution.
 #' @param ... force later arguments to be bound by name.
-#' @param subsMethod character, one of  c('stringsubs', 'langsubs')
+#' @param subsMethod character substitution method, one of  c('langsubs', 'stringsubs', 'subsubs').
+#' @param eval logical if TRUE execute the re-mapped expression (else return it).
 #' @param debugPrint logical if TRUE print debugging information when in stringsubs mode.
-#' @return result of expr executed in calling environment
+#' @return result of expr executed in calling environment (or expression if eval==FALSE).
 #'
 #' @examples
 #'
@@ -231,70 +299,70 @@ letprep_lang <- function(alias, lexpr) {
 #'                 Species='setosa',
 #'                 rank=c(1,2))
 #'
-#' RANKCOLUMN <- NULL # optional, make sure marco target does not look like unbound variable.
-#' GROUPCOLUMN <- NULL # optional, make sure marco target does not look like unbound variable.
-#' mapping = list(RANKCOLUMN= 'rank', GROUPCOLUMN= 'Species')
-#' let(alias=mapping,
-#'     expr={
+#' RANKCOLUMN <- NULL # optional, make sure macro target does not look like unbound variable.
+#' GROUPCOLUMN <- NULL # optional, make sure macro target does not look like unbound variable.
+#' mapping = c(RANKCOLUMN= 'rank', GROUPCOLUMN= 'Species')
+#' let(alias = mapping,
+#'     expr = {
 #'        # Notice code here can be written in terms of known or concrete
 #'        # names "RANKCOLUMN" and "GROUPCOLUMN", but executes as if we
 #'        # had written mapping specified columns "rank" and "Species".
 #'
 #'        # restart ranks at zero.
 #'        dres <- d
-#'        dres$RANKCOLUMN <- dres$RANKCOLUMN - 1 # notice using $ not [[]]
+#'        dres$RANKCOLUMN <- dres$RANKCOLUMN - 1 # notice, using $ not [[]]
 #'
 #'        # confirm set of groups.
 #'        groups <- unique(d$GROUPCOLUMN)
-#'     })
+#'     },
+#'     debugPrint = TRUE
+#'     )
 #' print(groups)
 #' print(length(groups))
 #' print(dres)
 #'
-#' # In string substitution mode let can replace string contents:
-#' let(list(x='y'), 'x', subsMethod= 'stringsubs')
-#'
-#' # In langsubs mode it will not:
-#' let(list(x='y'), 'x', subsMethod= 'langsubs')
-#'
 #'
 #' @export
 let <- function(alias, expr,
                 ...,
-                subsMethod= 'stringsubs',
+                subsMethod= 'langsubs',
+                eval= TRUE,
                 debugPrint= FALSE) {
   exprQ <- substitute(expr)  # do this early before things enter local environment
   if(length(list(...))>0) {
     stop("wrapr::let unexpected arguments")
   }
-  allowedMethods <- c('stringsubs', 'langsubs')
+  allowedMethods <- c('langsubs', 'stringsubs', 'subsubs')
   if((!is.character(subsMethod)) ||
      (length(subsMethod)!=1) ||
      (!(subsMethod %in% allowedMethods))) {
     stop(paste("wrapr::let subsMethod must be one of:",
                paste(allowedMethods, collapse = ', ')))
   }
-  exprS <- NULL
-  # if(subsMethod=='subsubs') {
-  #   # substitute based solution, not working so commented out
-  #   aliasN <- lapply(prepareAlias(alias), as.name)
-  #   # exprS <- substitute(deparse(exprQ), aliasN) # doesn't work as substitute sees "exprQ"
-  #   exprS <- do.call(substitute, list(exprQ, aliasN))
-  #   # substitute also fails to rebind left-hand side values as in dplyr::mutate(d, NEWCOL = 7) list(NEWCOL='z')
-  #   # example find in tests/testthat/test_letl.R
-  # } else
-  if(subsMethod=='langsubs') {
-    # recursive language implementation.
-    # only replace matching symbols.
-    exprS <- letprep_lang(prepareAlias(alias),
-                          exprQ)
-  } else if(subsMethod=='stringsubs') {
-    # string substitution based implementation.
-    # Similar to \code{gtools::strmacro} by Gregory R. Warnes.
-    exprS <- letprep_str(alias, deparse(exprQ),
-                     debugPrint=debugPrint)
-  } else {
-    stop(paste("wrapr::let unexpected subsMethod '", subsMethod, "'"))
+  alias <- prepareAlias(alias)
+  exprS <- exprQ
+  if(length(alias)>0) {
+    if(subsMethod=='langsubs') {
+      # recursive language implementation.
+      # only replace matching symbols.
+      exprS <- letprep_lang(alias, exprQ)
+    } else if(subsMethod=='subsubs') {
+      # substitute based solution, does not bind left-hand sides
+      aliasN <- lapply(alias, as.name)
+      exprS <- do.call(substitute, list(exprQ, aliasN))
+    } else if(subsMethod=='stringsubs') {
+      # string substitution based implementation.
+      # Similar to \code{gtools::strmacro} by Gregory R. Warnes.
+      exprS <- letprep_str(alias, deparse(exprQ))
+    } else {
+      stop(paste("wrapr::let unexpected subsMethod '", subsMethod, "'"))
+    }
+  }
+  if(debugPrint) {
+    print(exprS)
+  }
+  if(!eval) {
+    return(exprS)
   }
   # try to execute expression in parent environment
   rm(list=setdiff(ls(),'exprS'))
